﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
//using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using test;
using System.Text.RegularExpressions;

namespace Paging
{
    public partial class Paging : UserControl
    {
        #region 构造函数
        
        public Paging()
        {
            InitializeComponent();
        }

        #endregion

        #region 属性
        
        gate g = null;               //委托
        public  int pageIndex;       //当前页数
        private int pageMaxIndex;    //总页数
        public  int pageSize;        //每页显示记录数
        public  int pageCount;       //总记录数

        #endregion

        #region 外部调用
        
        /// <summary>
        /// 外部调用，委托方法
        /// </summary>
        public void External(gate g)
        {
            this.g = g;
        }

        /// <summary>
        /// 初始化，第一页
        /// </summary>
        public void Inital()
        {
            pageIndex = 1;
            pageSize = Convert.ToInt32(cmbPageSize.Text);
            UpdatePage();

            //如果一共只有一页，则所有控件不可用
            if (pageMaxIndex == 1)
            {
                foreach (Control con in this.Controls)
                {
                    con.Enabled = false;
                }
            }
            else
            {
                //控件之间互动
                btnPageUp.Enabled = false;
                btnHomePage.Enabled = false;
                btnNextPage.Enabled = true;
                btnLastPage.Enabled = true;
                btnJump.Enabled = true;
                txtJump.Enabled = true;
            }
        }

        #endregion

        #region 加载和公用方法
        
        /// <summary>
        /// 控件加载
        /// </summary>
        private void Paging_Load(object sender, EventArgs e)
        {
            pageSize = Convert.ToInt32(cmbPageSize.Text);
            PageMaxIndex();
        }
        
        /// <summary>
        /// 总页数计算
        /// </summary>
        private void PageMaxIndex()
        {
            if ((pageCount % pageSize) == 0)
                pageMaxIndex = (int)(pageCount / pageSize);
            else
                pageMaxIndex = (int)(pageCount / pageSize) + 1;
            lblPage.Text = pageIndex + "/" + pageMaxIndex;
        }

        /// <summary>
        /// 更新页面数据
        /// </summary>
        private void UpdatePage()
        {
            g(this);
            PageMaxIndex();
        }

        #endregion

        #region 控件事件
        
        /// <summary>
        /// 首页
        /// </summary>
        private void btnHomePage_Click(object sender, EventArgs e)
        {
            pageIndex = 1;
            UpdatePage();

            //控件之间互动
            btnPageUp.Enabled = false;
            btnNextPage.Enabled = true;
            btnLastPage.Enabled = true;
            btnJump.Enabled = true;
            txtJump.Enabled = true;
        }

        /// <summary>
        /// 上一页
        /// </summary>
        private void btnPageUp_Click(object sender, EventArgs e)
        {
            pageIndex--;
            UpdatePage();

            //控件之间互动
            //如果已经是第一页，则上一页、首页不可用
            if (pageIndex == 1)
            {
                btnPageUp.Enabled = false;
                btnHomePage.Enabled = false;
            }
            btnNextPage.Enabled = true;
            btnLastPage.Enabled = true;
        }

        /// <summary>
        /// 下一页
        /// </summary>
        private void btnNextPage_Click(object sender, EventArgs e)
        {
            pageIndex++;
            UpdatePage();

            //控件之间互动
            //如果已经是最后一页，则下一页、末页不可用
            if (pageIndex == pageMaxIndex)
            {
                btnNextPage.Enabled = false;
                btnLastPage.Enabled = false;
            }
            btnPageUp.Enabled = true;
            btnHomePage.Enabled = true;
        }

        /// <summary>
        /// 末页
        /// </summary>
        private void btnLastPage_Click(object sender, EventArgs e)
        {
            pageIndex = pageMaxIndex;
            UpdatePage();

            //控件之间互动
            btnNextPage.Enabled = false;
            btnPageUp.Enabled = true;
            btnHomePage.Enabled = true;
        }

        /// <summary>
        /// 跳转
        /// </summary>
        private void btnJump_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtJump.Text))
            {
                pageIndex = Convert.ToInt32(txtJump.Text);
                UpdatePage();
            }
        }

        /// <summary>
        /// 每页显示记录数改变时
        /// </summary>
        private void cmbPageSize_TextChanged(object sender, EventArgs e)
        {
            //验证输入是否合法，为正整数
            //正则表达式，匹配正整数 ，不满足则默认为“5”
            Regex regex = new Regex("^[0-9]*[1-9][0-9]*$");
            if (regex.IsMatch(cmbPageSize.Text))
            {
                //当记录数更改时，默认显示第一页
                pageSize = Convert.ToInt32(cmbPageSize.Text);
                pageIndex = 1;
                UpdatePage();

                //如果一共只有一页
                if (pageMaxIndex == 1)
                {
                    //控件之间互动
                    btnNextPage.Enabled = false;
                    btnLastPage.Enabled = false;
                    btnHomePage.Enabled = false;
                    btnPageUp.Enabled = false;
                    btnJump.Enabled = false;
                    txtJump.Enabled = false;
                }
                else
                {
                    //控件之间互动
                    btnPageUp.Enabled = false;
                    btnNextPage.Enabled = true;
                    btnLastPage.Enabled = true;
                    btnJump.Enabled = true;
                    txtJump.Enabled = true;
                }
            }
            else
                cmbPageSize.Text = "5";
        }

        /// <summary>
        /// 跳转页面页数改变时
        /// </summary>
        private void txtJump_TextChanged(object sender, EventArgs e)
        {
            //验证输入是否合法，0到pageMaxIndex之间整数
            //正则表达式，匹配正整数，且小于pageMaxIndex,否则删除刚刚输入的字符
            Regex regex = new Regex("^[0-9]*[1-9][0-9]*$");

            if (regex.IsMatch(txtJump.Text))
            {
                if (Convert.ToInt32(txtJump.Text) > pageMaxIndex)
                {
                    txtJump.Text = txtJump.Text.Substring(0, txtJump.Text.Length - 1);
                    txtJump.SelectionStart = txtJump.Text.Length;
                }
            }
            else
            {
                if (txtJump.Text.Length > 0)
                {
                    txtJump.Text = txtJump.Text.Substring(0, txtJump.Text.Length - 1);
                    txtJump.SelectionStart = txtJump.Text.Length;
                }
                else
                    txtJump.Text = "";

            }
        }

        #endregion
    }
}
